@@ -1,13 +1,11 @@
 /* apt-dater - terminal-based remote package update manager
- *
- * $Id: adproto.h 582 2011-09-27 13:45:43Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -37,6 +35,7 @@
 #define ADP_VERSION_0_3		0.3
 #define ADP_VERSION_0_4		0.4
 #define ADP_VERSION_0_5		0.5
+#define ADP_VERSION_0_6		0.6
 
 /* Most current protocol version. */
 #define ADP_CURRENT_VERSION	ADP_VERSION_0_5
@@ -61,6 +60,9 @@
 /* Features since protocol version 0.5 */
 #define ADP_FEATVER_ADPERR	ADP_VERSION_0_5
 
+/* Features since protocol version 0.6 */
+#define ADP_FEATVER_CLUSTER	ADP_VERSION_0_6
+
 
 /* Patterns */
 #define ADP_STRLEN_ADPERR	255
@@ -82,4 +84,10 @@
 #define ADP_STRLEN_VIRT		255
 #define ADP_PATTERN_VIRT	"VIRT: %"ADP_QUOTE(ADP_STRLEN_VIRT)"s"
 
+#define ADP_STRLEN_CLUSTER	255
+#define ADP_PATTERN_CLUSTER	"CLUSTER: %"ADP_QUOTE(ADP_STRLEN_CLUSTER)"s"
+
+#define ADP_STRLEN_ADPERR	255
+#define ADP_PATTERN_ADPERR	"ADPERR: %"ADP_QUOTE(ADP_STRLEN_ADPERR)"s"
+
 #endif /* _ADPROTO_H */
