@@ -1,13 +1,11 @@
 /* apt-dater - terminal-based remote package update manager
- *
- * $Id: env.c 582 2011-09-27 13:45:43Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2009-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -70,7 +68,11 @@ env_init(gchar **envp) {
 
 gchar **
 env_build(HostNode *n, const gchar *action, const gchar *param, const HistoryEntry *he) {
-    gchar **new_env = (gchar **) g_new0(gchar**, g_slist_length(base_env) + 20);
+    gchar **new_env = (gchar **) g_new0(gchar**, g_slist_length(base_env) + 20
+#ifdef FEAT_CLUSTERS
+    + 1 + g_list_length(n->clusters)
+#endif
+    );
     gint i = 0;
 
     GSList *p;
@@ -122,6 +124,20 @@ env_build(HostNode *n, const gchar *action, const gchar *param, const HistoryEnt
 #endif
      ADD_HENV("HIST_PATH"       , "");
 
+#ifdef FEAT_CLUSTERS
+    if(n->clusters != NULL) {
+	new_env[i++] = g_strdup_printf("AD_CLUSTERS=%d", g_list_length(n->clusters));
+	int j = 1;
+	GList *c = n->clusters;
+	while(c != NULL) {
+	    new_env[i++] = g_strdup_printf("AD_CLUSTER%d=%s", j, (gchar *)c->data);
+	    c = c->next;
+	    j++;
+	}
+    }
+    else
+#endif
+     ADD_HENV("CLUSTERS"        , "0");
 
     ADD_HENV("ACTION"           , action);
     if(param)
