@@ -1,13 +1,11 @@
 /* apt-dater - terminal-based remote package update manager
- *
- * $Id: stats.c 582 2011-09-27 13:45:43Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -25,7 +23,7 @@
  *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-#include <glib-2.0/glib/gstdio.h>
+#include <glib/gstdio.h>
 
 #include "apt-dater.h"
 #include "exec.h"
@@ -33,6 +31,7 @@
 #include "stats.h"
 #include "lock.h"
 #include "autoref.h"
+#include "clusters.h"
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -96,6 +95,9 @@ gboolean setStatsFileFromIOC(GIOChannel *ioc, GIOCondition condition,
     if(iostatus == G_IO_STATUS_ERROR || iostatus == G_IO_STATUS_AGAIN)
      break;
 
+    if(*buf == 0)
+    	condition = G_IO_HUP;
+
     if(fwrite(buf, sizeof(gchar), bytes, ((HostNode *) n)->fpstat) != bytes)
 	break;
 
@@ -219,11 +221,19 @@ gboolean getUpdatesFromStat(HostNode *n)
   g_free(n->virt);
   n->virt = NULL;
  }
+ if(n->adperr) {
+  g_free(n->adperr);
+  n->adperr = NULL;
+ }
  if(n->kernelrel) {
   g_free(n->kernelrel);
   n->kernelrel = NULL;
  }
 
+#ifdef FEAT_CLUSTERS
+  cluster_host_reset(n);
+#endif
+
  int linesok = 0;
  while(fgets(line, STATS_MAX_LINE_LEN, fp)) {
   /* Remove any whitespace from the line. */
@@ -248,12 +258,6 @@ gboolean getUpdatesFromStat(HostNode *n)
     continue;
   }
 
-// ADP_PATTERN_ADPERR
-  if(!strncmp("ADPERR: ", line, 8)) {
-    adperr = TRUE;
-    continue;
-  }
-
   if(!strncmp("STATUS: ", line, 8)) {
     argv = g_strsplit(&line[8], "|", 0);
 
@@ -352,6 +356,12 @@ gboolean getUpdatesFromStat(HostNode *n)
    continue;
   }
 
+  if (sscanf((gchar *) line, ADP_PATTERN_ADPERR, buf)) {
+   n->adperr = g_strdup(buf);
+   adperr = TRUE;
+   continue;
+  }
+
   if (sscanf((gchar *) line, ADP_PATTERN_UNAME, buf)) {
    gchar *s = strchr(buf, '|');
    if (s) {
@@ -369,11 +379,24 @@ gboolean getUpdatesFromStat(HostNode *n)
    continue;
 
   if (sscanf((gchar *) line, "UUID: %" QUOTE(UUID_STRLEN) "s", n->uuid)) {
-   n->uuid[UUID_STRLEN] = 0;
+   n->uuid[UUID_STRLEN-1] = 0;
 
    linesok++;
    continue;
   }
+
+#ifdef FEAT_CLUSTERS
+  char cluster[ADP_STRLEN_CLUSTER];
+  if (sscanf((gchar *) line, ADP_PATTERN_CLUSTER, cluster)) {
+   cluster[ADP_STRLEN_CLUSTER-1] = 0;
+
+   cluster_host_add(n, cluster);
+
+   linesok++;
+   continue;
+  }
+#endif
+
  }
 
  if((!adproto && linesok > 10) ||
@@ -397,6 +420,14 @@ gboolean getUpdatesFromStat(HostNode *n)
  return(TRUE);
 }
 
+gchar *getStatsContent(const HostNode *n) {
+ gchar *c = NULL;
+
+ if(!g_file_get_contents(n->statsfile, &c, NULL, NULL))
+    return NULL;
+
+ return c;
+}
 
 gboolean refreshStats(GList *hosts)
 {
