@@ -1,13 +1,11 @@
 /* apt-dater - terminal-based remote package update manager
- *
- * $Id: stats.h 582 2011-09-27 13:45:43Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -28,7 +26,7 @@
 #ifndef _STATS_H
 #define _STATS_H
 
-#include <glib-2.0/glib.h>
+#include <glib.h>
 #include "apt-dater.h"
 
 void
@@ -40,8 +38,9 @@ freeUpdates(GList *updates);
 gboolean refreshStats(GList *);
 gboolean setStatsFileFromIOC(GIOChannel *, GIOCondition, gpointer);
 gchar *getStatsFileName(const HostNode *);
-gboolean prepareStatsFile(HostNode *n);
+gboolean prepareStatsFile(HostNode *);
 gboolean getUpdatesFromStat(HostNode *);
 void refreshStatsOfNode(gpointer);
+gchar *getStatsContent(const HostNode *);
 
 #endif /* _STATS_H */
