@@ -1,13 +1,11 @@
 /* apt-dater - terminal-based remote package update manager
- *
- * $Id: ui.c 582 2011-09-27 13:45:43Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -31,6 +29,7 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <termios.h>
+#include <string.h>
 #include "apt-dater.h"
 #include "ui.h"
 #include "colors.h"
@@ -99,6 +98,9 @@ typedef enum {
     TCLMK_VIRT,
     TCLMK_FORBID,
 
+#ifdef FEAT_CLUSTERS
+    TCLMK_CLUSTERS,
+#endif
     TCLMK_EXTRAS,
     TCLMK_FLAGS,
     TCLMK_HOLDS,
@@ -124,6 +126,9 @@ const static struct TCLMapping tclmap[] = {
     {TCLMK_VIRT     , "virt"      , TCLM_STRING},
     {TCLMK_FORBID   , "forbid"    , TCLM_INT},
 
+#ifdef FEAT_CLUSTERS
+    {TCLMK_CLUSTERS , "clusters"  , TCLM_IGNORE},
+#endif
     {TCLMK_EXTRAS   , "extras"    , TCLM_IGNORE},
     {TCLMK_FLAGS    , "flags"     , TCLM_IGNORE},
     {TCLMK_HOLDS    , "holds"     , TCLM_IGNORE},
@@ -172,6 +177,7 @@ static struct ShortCut shortCuts[] = {
  {SC_KEY_FILETRANS, 'C', "C" , N_("file transfer") , FALSE, 0},
  {SC_KEY_TOGGLEDUMPS, 'd', "d" , N_("toggle dumps") , FALSE, VK_DUMP},
  {SC_KEY_REFRESH, 'g', "g" , N_("refresh host") , FALSE, VK_REFRESH},
+ {SC_KEY_ERRDIAG, 'e', "e" , N_("failure diagnostic") , FALSE, VK_ERRDIAG},
  {SC_KEY_INSTALL, 'i', "i" , N_("install pkg") , FALSE, VK_INSTALL},
  {SC_KEY_UPGRADE, 'u', "u" , N_("upgrade host(s)") , FALSE, VK_UPGRADE},
  {SC_KEY_MORE, 'm', "m" , N_("host details") , FALSE, 0},
@@ -202,6 +208,9 @@ const struct HostFlag hostFlags[] = {
   {HOST_STATUS_KERNELNOTMATCH ,  "R", N_("running kernel is not the latest (reboot required)")},
   {HOST_STATUS_KERNELSELFBUILD,  "k", N_("a selfbuild kernel is running")},
   {HOST_STATUS_VIRTUALIZED    ,  "v", N_("this is a virtualized machine")},
+#ifdef FEAT_CLUSTERS
+  {HOST_STATUS_CLUSTERED      ,  "C", N_("this machine is part of a cluster")},
+#endif
   {0                          , NULL, NULL},
 };
 
@@ -440,7 +449,7 @@ void setHostsCategory(GList *hosts, Category matchCategory, gchar *matchGroup, C
 
  while(ho) {
   if(matchGroup) {
-   if((((HostNode *) ho->data)->category == matchCategory) && (!g_strcasecmp(((HostNode *) ho->data)->group, matchGroup))) ((HostNode *) ho->data)->category = setCategory;
+   if((((HostNode *) ho->data)->category == matchCategory) && (!g_ascii_strcasecmp(((HostNode *) ho->data)->group, matchGroup))) ((HostNode *) ho->data)->category = setCategory;
   }
   else if((((HostNode *) ho->data)->category == matchCategory)) ((HostNode *) ho->data)->category = setCategory;
   ho = g_list_next(ho);
@@ -457,11 +466,11 @@ guint getHostGrpCatCnt(GList *hosts, gchar *group, Category category)
  while(ho) {
 #ifdef FEAT_TCLFILTER
   if(category == C_FILTERED) {
-    if((((HostNode *) ho->data)->filtered) && (!g_strcasecmp(((HostNode *) ho->data)->group, group))) cnt++;
+    if((((HostNode *) ho->data)->filtered) && (!g_ascii_strcasecmp(((HostNode *) ho->data)->group, group))) cnt++;
   }
   else
 #endif
-   if((((HostNode *) ho->data)->category == category) && (!g_strcasecmp(((HostNode *) ho->data)->group, group))) cnt++;
+   if((((HostNode *) ho->data)->category == category) && (!g_ascii_strcasecmp(((HostNode *) ho->data)->group, group))) cnt++;
 
   ho = g_list_next(ho);
  }
@@ -477,7 +486,7 @@ guint getHostGrpCatTaggedCnt(GList *hosts, const gchar *group, Category category
  ho = g_list_first(hosts);
 
  while(ho) {
-  if((((HostNode *) ho->data)->category == category) && (!g_strcasecmp(((HostNode *) ho->data)->group, group)) && ((HostNode *) ho->data)->tagged == TRUE) cnt++;
+  if((((HostNode *) ho->data)->category == category) && (!g_ascii_strcasecmp(((HostNode *) ho->data)->group, group)) && ((HostNode *) ho->data)->tagged == TRUE) cnt++;
 
   ho = g_list_next(ho);
  }
@@ -716,6 +725,19 @@ static void drawHostDetails(HostNode *h)
    mvwaddnstr(wp, l-1, 21 + strlen(h->kernelrel), buf, COLS - 21 - strlen(h->kernelrel));
  }
 
+#ifdef FEAT_CLUSTERS
+ if(g_list_length(h->clusters)) {
+  l++;
+
+  mvwaddnstr(wp, l  , 2, _("Clusters: "), COLS - 2);
+  GList *c = h->clusters;
+  while(c) {
+   mvwaddnstr(wp, l++, 20, c->data, COLS - 20);
+   c = g_list_next(c);
+  }
+ }
+#endif
+
  if (g_list_length(h->packages)) {
   l++;
 
@@ -854,6 +876,62 @@ static void drawHostDetails(HostNode *h)
 }
 
 
+static void drawHostErrDiag(HostNode *h)
+{
+ gchar *buf = getStatsContent(h);
+ int l = 2;
+
+ if(buf != NULL) {
+  gchar *b = buf;
+  while((b = strchr(b, '\n'))) {
+    l++;
+    b++;
+  }
+
+  b = buf;
+  while((b = strchr(b, '\r'))) {
+    *b = ' ';
+  }
+ }
+
+ WINDOW *wp = newpad(MAX(LINES, l+2), COLS);
+ int  wic = 0, pminrow = 0, kcquit = 'q', i;
+
+ keypad(wp, TRUE);
+
+ wattron(wp, A_BOLD);
+ mvwaddnstr(wp, 0,  1, _("FAILURE DIAGNOSTIC"), COLS - 1);
+ wattroff(wp, A_BOLD);
+
+ mvwaddstr(wp, 2,  0, buf);
+ g_free(buf);
+
+ for(i = 0; shortCuts[i].key; i++)
+  if(shortCuts[i].sc == SC_KEY_QUIT) kcquit = shortCuts[i].keycode;
+
+ prefresh(wp, pminrow, 0, 1, 0, LINES-3, COLS);
+ while((wic = tolower(wgetch(wp))) != kcquit) {
+  if(wic == KEY_UP && pminrow)
+   pminrow--;
+  else if(wic == KEY_DOWN && pminrow < l-LINES+4)
+   pminrow++;
+  else if(wic == KEY_HOME)
+   pminrow=0;
+  else if(wic == KEY_END)
+   pminrow=l-LINES+4;
+  else if(wic == KEY_NPAGE)
+   pminrow = pminrow+LINES-3 > (l-LINES+4) ? l-LINES+4 : pminrow+LINES-3;
+  else if(wic == KEY_PPAGE)
+   pminrow = pminrow-(LINES-3) < 0 ? 0 : pminrow-(LINES-3);
+  else if(wic == '?')
+   drawHelp();
+  prefresh(wp, pminrow, 0, 1, 0, LINES-3, COLS);
+ }
+
+ delwin(wp);
+}
+
+
 void drawStatus (char *str, gboolean drawinfo)
 {
  char strinfo[30];
@@ -1246,7 +1324,11 @@ void drawHostEntry (DrawNode *n)
 
   attroff(uicolors[UI_COLOR_HOSTSTATUS]);
  }
+#ifdef FEAT_CLUSTERS
+ mvaddstr(n->scrpos, 7, " [");
+#else
  mvaddstr(n->scrpos, 6, " [");
+#endif
 
  if(n->elements > 0)
   addch(n->elements > 0 && n->extended == FALSE ? '+' : '-');
@@ -1341,8 +1423,13 @@ void drawHostEntry (DrawNode *n)
    }
    break;
   default:
-   mask = VK_CONNECT | VK_REFRESH | VK_INSTALL;
-   sprintf(statusln, _("Status is unknown"));
+   mask = VK_CONNECT | VK_REFRESH | VK_INSTALL | VK_ERRDIAG;
+   if(((HostNode *) n->p)->adperr == NULL) {
+    sprintf(statusln, _("Status is unknown"));
+   }
+   else {
+    sprintf(statusln, _("Error: %s"), ((HostNode *) n->p)->adperr);
+   }
    break;
   }
 
@@ -1870,16 +1957,16 @@ gboolean compDrawNodes(DrawNode* n1, DrawNode* n2)
  switch(n1->type) {
  case CATEGORY:
  case GROUP:
-  if(!g_strcasecmp(n1->p, n2->p))
+  if(!g_ascii_strcasecmp(n1->p, n2->p))
    return(TRUE);
   break;
  case HOST:
-  if (!g_strcasecmp(((HostNode *) (n1->p))->hostname, 
+  if (!g_ascii_strcasecmp(((HostNode *) (n1->p))->hostname, 
 		    ((HostNode *)(n2->p))->hostname))
    return(TRUE);
   break;
  case PKG:
-  if (!g_strcasecmp(((PkgNode *) n1->p)->package,
+  if (!g_ascii_strcasecmp(((PkgNode *) n1->p)->package,
 		    ((PkgNode *) n2->p)->package)) return(TRUE);
   break;
  case SESSION:
@@ -2025,7 +2112,7 @@ void extDrawListCategory(gint atpos, gchar *category, GList *hosts)
 #endif
      (((HostNode *) ho->data)->category == i)
     ) {
-   if((drawnode) && (!g_strcasecmp(drawnode->p, 
+   if((drawnode) && (!g_ascii_strcasecmp(drawnode->p, 
 				   ((HostNode *) ho->data)->group))) {
     if(((HostNode *) ho->data)->tagged == TRUE) drawnode->etagged++;
     ho = g_list_next(ho);
@@ -2065,7 +2152,7 @@ void extDrawListGroup(gint atpos, gchar *group, GList *hosts)
  parent = g_list_nth_data(drawlist, (guint) atpos);
 
  while(ho) {
-  if(!g_strcasecmp(((HostNode *) ho->data)->group, group) && 
+  if(!g_ascii_strcasecmp(((HostNode *) ho->data)->group, group) && 
      (
 #ifdef FEAT_TCLFILTER
       (incategory == drawCategories[C_FILTERED] && ((HostNode *) ho->data)->filtered) ||
@@ -2707,13 +2794,25 @@ void applyFilter(GList *hosts) {
         Tcl_SetVar2(tcl_interp, "flags", hostFlags[i].code, hostFlags[i].code, 0);
     }
 
+#ifdef FEAT_CLUSTERS
+    Tcl_UnsetVar(tcl_interp, "clusters", 0);
+    GList *c = n->clusters;
+    while(c) {
+	Tcl_SetVar2(tcl_interp, "clusters", c->data, c->data, 0);
+	c = g_list_next(c);
+    }
+#endif
+
     Tcl_ResetResult(tcl_interp);
-    tcl_interp->errorLine = 0;
-    Tcl_Eval(tcl_interp, filterexp);
-    if(tcl_interp->errorLine)
-     filtered = FALSE;
-    else
-     filtered = atoi(tcl_interp->result) > 0;
+    switch(Tcl_Eval(tcl_interp, filterexp)) {
+	case TCL_OK:
+	case TCL_RETURN:
+	    filtered = atoi(Tcl_GetStringResult(tcl_interp)) > 0;
+	    ;;
+	default:
+	    filtered = FALSE;
+	    ;;
+    }
 
     if(filtered != n->filtered) {
       n->filtered = filtered;
@@ -3507,6 +3606,14 @@ gboolean ctrlUI (GList *hosts)
    }
    break; /* case SC_KEY_MORE */
 
+  case SC_KEY_ERRDIAG:
+   if (inhost) {
+    drawHostErrDiag(inhost);
+
+    refscr = TRUE;
+   }
+   break; /* case SC_KEY_ERRDIAG */
+
 #ifdef FEAT_HISTORY
   case SC_KEY_HISTORY:
    if (inhost) {
