@@ -1,13 +1,11 @@
 /* apt-dater - terminal-based remote package update manager
- *
- * $Id: ui.h 582 2011-09-27 13:45:43Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2012 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -35,7 +33,7 @@
 #include <curses.h>
 #endif
 
-#include <glib-2.0/glib.h>
+#include <glib.h>
 
 typedef enum {
  CATEGORY,
@@ -53,9 +51,10 @@ typedef enum {
  VK_INSTALL=16,
  VK_REFRESH=32,
  VK_KILL=64,
+ VK_ERRDIAG=128,
 #ifdef FEAT_HISTORY
- VK_PLAY=128,
- VK_LESS=256,
+ VK_PLAY=256,
+ VK_LESS=512,
 #endif
 } EVisKeyMask;
 
@@ -90,6 +89,7 @@ typedef enum {
  SC_KEY_INSTALL,
  SC_KEY_UPGRADE,
  SC_KEY_MORE,
+ SC_KEY_ERRDIAG,
 #ifdef FEAT_HISTORY
  SC_KEY_HISTORY,
  SC_KEY_PLAY,
